/**************************************************************************
*   Copyright (C) 2008 by Michael Hoffer <info@michaelhoffer.de>          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as               *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public             *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef CANVAS_H
#define CANVAS_H

#include <QWidget>

#include "ImageArray.h"
#include "ColorTable.h"
#include "ValueListener.h"

/**
 *
 * The canvas class.
 *
 * It is derived from the QCanvas class and will be used as custom
 * widget in the designer.
 */
class Canvas : public QWidget
{
		Q_OBJECT

	public:
		/**
		 * Constructor
		 */
		Canvas( QWidget *parent = NULL );

		void updatePreview();

		void setImageArray( ImageArray imageArray );
		void updateImageArray();
		void setColorTable( ColorTable colorTable );
		void paintEvent( QPaintEvent * event );

		bool isInitialized();

		ImageArray& getImageArray();
		QImage& getScaledImage();
		virtual void setScaledImage(QImage scaledImage);
		QImage& getMainImage();

		int getArrayValue(int x, int y);

	protected slots:
		/*
		 * These slots are named by their corresponding
		 * signals. This ensures they are automatically
		 * connected. Se Qt documentation for details.
		 */
	private slots:
		//void on_pbOK_clicked();
		//void on_pbStop_clicked();

		//void on_actionQuit_triggered();

	protected:
		ImageArray _imageArray;
		ColorTable _colorTable;
	private:

		QImage _mainImage;
		QImage _scaledImage;

		bool _initialized;

};
#endif /*CANVAS_H*/
