/**************************************************************************
*   Copyright (C) 2008 by Michael Hoffer <info@michaelhoffer.de>          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as               *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public             *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef ARRAY_H
#define ARRAY_H

#include<QImage>

#include"ColorTable.h"

#include <string>
#include <fstream>
#include "ImageReader.h"
#include "CbfReader.h"


/**
 *
 * ImageArray class.
 *
 */

class ImageArray
{
	public:
		ImageArray();
		/**
		 * Constructor
		 */
		ImageArray( unsigned int nCols, unsigned int nRows );
		ImageArray( const ImageArray &imageArray );
		ImageArray( std::string fileName );

		ImageArray& operator= ( const ImageArray& imageArray );
		const int& operator()( unsigned int x, unsigned int y ) const;
		int& operator()( unsigned int x, unsigned int y );

		void zero();

		const unsigned int nCols() const;
		const unsigned int nRows() const;

		const bool isInitialized() const;

		const int getMinValue();
		const int getMaxValue();

		const int getSlope() const;
		void setSlope(int slope);

		void computeColorMap();

		/**
		 * if we use this array inside of color canvas
		 * it is necessary to use the color map from
		 * the corresponding imagearray that contains the image
		 * data
		 */
		void setCustomColorMap(ImageArray &img);

		const unsigned int getColorIndexValue( int i );

		QImage toQImage();

		~ImageArray();


	private:
		int* _memory;
		unsigned int _nCols;
		unsigned int _nRows;

		int _minValue;
		int _maxValue;

		bool _limitsOutOfDate;

		double _step;
		int _slope;
		int _percentile;

		bool _customColorMap;

		std::vector<ImageReader*> _imageReaders;


		void computeLimits();

		int pix_percentile( float p );

};
#endif /*ARRAY_H*/
