/**************************************************************************
*   Copyright (C) 2008 by Michael Hoffer <info@michaelhoffer.de>          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as               *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public             *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "ImageReader.h"

ImageReader::ImageReader()
{
	_nx = 0;
	_ny = 0;
	_memory = NULL;
}

const unsigned int ImageReader::getNX() const
{
	return _nx;
}

const unsigned int ImageReader::getNY() const
{
	return _ny;
}

int* ImageReader::getMemory() const
{
	return _memory;
}

void ImageReader::setMemory(int* memory){
	_memory = memory;
}

int ImageReader::stream2String( std::ifstream& tapin, std::string &out )
{
	std::string line;

	bool markerFound = false;
	while ( getline( tapin, line ) )
	{
		out += line + "\n";
		if ( line.size() == 1 )
		{
			if ( isMarker( tapin ) == true )
			{
				markerFound = true;
				break;
			}
		}
	}

	if ( markerFound == false )
	{
		std::cerr << "binary marker not found!" << std::endl;
		return -3; // Wrong image format
	}

	return 0;
}

std::vector<std::string> ImageReader::tokenize( std::string in, char seperator )
{
	std::stringstream stream( in );
	std::vector<std::string> tokens;
	std::string buffer;

	while ( getline( stream, buffer, seperator ) )
	{
		tokens.push_back( buffer );
	}
	return tokens;
}

std::string ImageReader::toUpper( std::string str )
{
	std::transform( str.begin(), str.end(), str.begin(), 
			( int( * )( int ) )std::toupper );
	return str;
}

int ImageReader::getTokenID( std::string s, std::vector<std::string> token )
{
	int result = -1;
	for ( unsigned int i = 0; i < token.size(); i++ )
	{
		if ( toUpper( s ) == toUpper( token[i] ) )
		{
			result = i;
			break;
		}
	}

	return result;
}

std::string ImageReader::removeWhiteSpace( std::string in, char whiteSpace )
{
	std::string out;
	for ( unsigned int i = 0; i < in.size(); i++ )
	{
		if ( in[i] != whiteSpace )
		{
			out += in[i];
		}
	}
	return out;
}

bool ImageReader::findToken( std::string token, 
		std::vector<std::string> tokenList,
		bool whiteSpaces )
{
	bool result = false;
	//std::cout << "<FINDTOKEN>\n";
	for ( unsigned int i = 0; i < tokenList.size();i++ )
	{
		tokenList[i] = removeWhiteSpace( tokenList[i], '\n' );
		tokenList[i] = removeWhiteSpace( tokenList[i], '\r' );

		if ( whiteSpaces )
		{
			tokenList[i] = removeWhiteSpace( tokenList[i], ' ' );
			tokenList[i] = removeWhiteSpace( tokenList[i], '\t' );
			//tokenList[i] = removeWhiteSpace( tokenList[i], '\"' );

			token = removeWhiteSpace( token, ' ' );
			token = removeWhiteSpace( token, '\t' );
		}

		//std::cout << ">> DEBUG: TOKENLIST: " << tokenList[i] << std::endl;
		//std::cout << ">> DEBUG: TOKEN: " << token << std::endl;


		if ( toUpper( token ) == toUpper( tokenList[i] ) )
		{
			result = true;
			break;
		}
	}

	//std::cout << "</FINDTOKEN>\n";
	return result;
}

