/**************************************************************************
*   Copyright (C) 2008 by Michael Hoffer <info@michaelhoffer.de>          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as               *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public             *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "ViewCanvas.h"
#include <QPainter>

#include <iostream>
#include <cmath>

ViewCanvas::ViewCanvas( QWidget *parent ) : Canvas( parent )
{
	_listener = NULL;

	setMouseTracking( true );

	_zoomRect = QRect( 0, 0, 0, 0 );
	_viewRect = QRect( 0, 0, 0, 0 );

	_zoomRect1 = new QRubberBand( QRubberBand::Rectangle, this );

	_array2MouseStepX = 0;
	_array2MouseStepY = 0;

	_mouse2ArrayStepX = 0;
	_mouse2ArrayStepY = 0;

	_mouse2ArrayOffsetX = 0;
	_mouse2ArrayOffsetY = 0;

	_zoomViewStack = QVector<QRect>();

	_imageBuffer = QImage( this->width(), this->height(), QImage::Format_RGB32 );
}

void ViewCanvas::setListener( ValueListener *listener )
{
	_listener = listener;
}

void ViewCanvas::mousePressEvent( QMouseEvent *e )
{
	//-------------------
	// initiating buttons
	//-------------------
	if ( e->button() == Qt::LeftButton )
	{
		_zoomViewStack.push_front( _viewRect );

		double arrayX = floor( _mouse2ArrayStepX * e->x() + _mouse2ArrayOffsetX );
		double arrayY = floor( _mouse2ArrayStepY * e->y() + _mouse2ArrayOffsetY );

		int x = ( int )floor( _array2MouseStepX * ( arrayX - _mouse2ArrayOffsetX ) + 1 );
		int y = ( int )floor( _array2MouseStepY * ( arrayY - _mouse2ArrayOffsetY ) + 1 );

		_zoomRect.setX( x );
		_zoomRect.setY( y );

		_zoomRect.setWidth( 0 );
		_zoomRect.setHeight( 0 );

		this->repaint();
	}

	if ( e->button() == Qt::MidButton )
	{
		_zoomViewStack.push_front( _viewRect );

		double arrayX = floor( _mouse2ArrayStepX * e->x() + _mouse2ArrayOffsetX );
		double arrayY = floor( _mouse2ArrayStepY * e->y() + _mouse2ArrayOffsetY );

		int x = ( int )floor( _array2MouseStepX * ( arrayX - _mouse2ArrayOffsetX ) + 1 );
		int y = ( int )floor( _array2MouseStepY * ( arrayY - _mouse2ArrayOffsetY ) + 1 );

		_zoomRect.setWidth( 0 );
		_zoomRect.setHeight( 0 );

		_zoomRect.setX( x );
		_zoomRect.setY( y );

	}

}

bool ViewCanvas::computeViewRect( int x, int y )
{
	int x0 = 0;
	int y0 = 0;

	int offsetX = _viewRect.width() / 2;
	int offsetY = _viewRect.height() / 2;

	int xTmp = x - offsetX;
	int yTmp = y - offsetY;

	QPoint viewRectPos;

	bool result = false;

	if ( xTmp >= 0 && yTmp >= 0 )
	{
		// quadrant 1
		x0 = _viewRect.width();
		y0 = _viewRect.height();

		viewRectPos = QPoint( x0 - ( x0 - x ) / 2 - offsetX, y0 - ( y0 - y ) / 2 - offsetY );
	}
	if ( xTmp < 0 && yTmp >= 0 )
	{
		// quadrant 2
		x0 = 0;
		y0 = _viewRect.height();

		viewRectPos = QPoint( x / 2, y0 - ( y0 - y ) / 2 - offsetY );
	}
	if ( xTmp < 0 && yTmp < 0 )
	{
		// quadrant 3
		x0 = 0;
		y0 = 0;

		viewRectPos = QPoint( x / 2, y / 2 );
	}
	if ( xTmp >= 0 && yTmp < 0 )
	{
		// quadrant 4
		x0 = _viewRect.width();
		y0 = 0;

		viewRectPos = QPoint( x0 - ( x0 - x ) / 2 - offsetX, y / 2 );
	}

	if ( offsetX > _mouse2ArrayStepX && offsetY > _mouse2ArrayStepY )
	{
		_viewRect.translate( viewRectPos );
		_viewRect.setWidth( offsetX );
		_viewRect.setHeight( offsetY );

		result = true;
	}


	return result;
}


void ViewCanvas::mouseReleaseEvent( QMouseEvent *e )
{
	if ( e->button() == Qt::LeftButton )
	{
		if (( _zoomRect.width() != 0 ) && ( _zoomRect.height() != 0 ) )
		{
			int x = ( int )floor( _mouse2ArrayStepX * _zoomRect.x() );
			int y = ( int )floor( _mouse2ArrayStepY * _zoomRect.y() );


			int width = ( int )ceil( _mouse2ArrayStepX * _zoomRect.width() );
			int height = ( int )ceil( _mouse2ArrayStepY * _zoomRect.height() );

			if ( width < 0 )
			{
				width = abs( width );
				x -= width;
			}

			if ( height < 0 )
			{
				height = abs( height );
				y -= height;
			}

			if ( width > 0 && height > 0 )
			{
				_viewRect.translate( x, y );

				_viewRect.setWidth( width );
				_viewRect.setHeight( height );

				initZoomValues();
				_zoomRect = QRect( 0, 0, 0, 0 );

			}
			//_zoomRect1->hide();
		}
	}
	if ( e->button() == Qt::MidButton )
	{
		int x = ( int )floor( _mouse2ArrayStepX * _zoomRect.x() );
		int y = ( int )floor( _mouse2ArrayStepY * _zoomRect.y() );

		if ( computeViewRect( x, y ) )
		{
			initZoomValues();
			_zoomRect = QRect( 0, 0, 0, 0 );
		}
		else
		{
			if ( _zoomViewStack.size() > 1 )
			{
				_zoomViewStack.pop_front();
			}
		}
		//_zoomRect1->hide();

	}
	if ( e->button() == Qt::RightButton )
	{
		_viewRect = QRect( 0, 0, getImageArray().nCols(), getImageArray().nRows() );

		_viewRect = _zoomViewStack.first();

		if ( _zoomViewStack.size() > 1 )
		{
			_zoomViewStack.pop_front();
		}

		initZoomValues();
	}

	this->repaint();
}

void ViewCanvas::mouseMoveEvent( QMouseEvent * e )
{
	if ( isInitialized() )
	{
		//-------------------
		// initiating buttons
		//-------------------
		//if ( e->button() == Qt::LeftButton )
		//{
		//int x = getImageArray().nCols() / this->width() * e->x();
		//int y = getImageArray().nRows() / this->height() * e->y();

		//        int x = e->x();
		//        int y = e->y();

		//        _zoomRect.setX( x );
		//        _zoomRect.setY( y );

		//        _listener->valueChanged( 0, x, y );
		//}
		if ( e->button() == Qt::RightButton )
		{
			//
		}

		//--------------------
		// active buttons
		//--------------------

		if ( e->buttons() == Qt::LeftButton )
		{
			int diffX = e->x() - _zoomRect.x();
			int diffY = e->y() - _zoomRect.y();

			if (( abs( diffX ) >= _array2MouseStepX / 2.0 )
			        && ( abs( diffY ) >= _array2MouseStepY / 2.0 ) )
			{
				double arrayDiffX = round( _mouse2ArrayStepX * diffX );
				double arrayDiffY = round( _mouse2ArrayStepY * diffY );

				diffX = ( int )ceil( _array2MouseStepX * arrayDiffX - 1 );
				diffY = ( int )ceil( _array2MouseStepY * arrayDiffY - 1 );

				_zoomRect.setWidth( diffX );
				_zoomRect.setHeight( diffY );

				_listener->valueChanged( 0, diffX, diffY );

				this->repaint();
			}
		}

		if ( e->buttons() == Qt::NoButton )
		{
			int value = 0;

			int x = ( int )floor( _mouse2ArrayStepX * e->x() + _mouse2ArrayOffsetX );
			int y = ( int )floor( _mouse2ArrayStepY * e->y() + _mouse2ArrayOffsetY );

			// Qt also receives mouse events from outside of
			// the arrays boundaries: these four if statements prevent that
			if ( x >= getImageArray().nCols() )
			{
				x = getImageArray().nCols() - 1;
			}

			if ( y >= getImageArray().nRows() )
			{
				y = getImageArray().nRows() - 1;
			}

			if ( x < 0 )
			{
				x = 0;
			}

			if ( y < 0 )
			{
				y = 0;
			}

			if ( _listener != NULL )
			{
				_listener->valueChanged( getImageArray()( x, y ), x , y );
			}
		}
	}
}

void ViewCanvas::setImageArray( ImageArray imageArray )
{
	Canvas::setImageArray( imageArray );

	_viewRect = QRect( 0, 0, imageArray.nCols(), imageArray.nRows() );
	_zoomRect = QRect( 0, 0, 0, 0 );

	_zoomViewStack = QVector<QRect>();
	_zoomViewStack.push_front( _viewRect );
	initZoomValues();
}

void ViewCanvas::initZoomValues()
{
	_mouse2ArrayStepX = ( double )_viewRect.width() / this->width();
	_mouse2ArrayStepY = ( double )_viewRect.height() / this->height();

	_array2MouseStepX = ( double )this->width() / _viewRect.width();
	_array2MouseStepY = ( double )this->height() / _viewRect.height();

	_mouse2ArrayOffsetX = _viewRect.x();
	_mouse2ArrayOffsetY = _viewRect.y();

	_zoomRect = QRect( 0, 0, 0, 0 );

	updateImageBuffer();
}

void ViewCanvas::paintEvent( QPaintEvent * event )
{
	if ( isInitialized() )
	{
		QPainter p( this );

		QRect target( 0, 0, this->width(), this->height() );

		p.drawImage( target, _imageBuffer );
	}

	QPainter p( this );

	QPen pen;
	pen.setColor( Qt::blue );
	pen.setWidth( 2 );

	pen.setStyle( Qt::DashDotLine );

	p.setPen( pen );
	QRect paintRect = _zoomRect;

	if (( paintRect.width() != 0 ) && ( paintRect.height() != 0 ) )
	{
		paintRect.setWidth( paintRect.width() - 1 );
		paintRect.setHeight( paintRect.height() - 1 );

		p.drawRect( paintRect );
	}
}

void ViewCanvas::updateImageBuffer()
{
	QPainter p( &_imageBuffer );
	QRect target( 0, 0, this->width(), this->height() );
	p.drawImage( target, getMainImage(), _viewRect );
}

void ViewCanvas::resizeEvent( QResizeEvent *event )
{
	initZoomValues();
	_imageBuffer = QImage( this->width(), this->height(), QImage::Format_RGB32 );
	updateImageBuffer();
}
