/**************************************************************************
*   Copyright (C) 2008 by Michael Hoffer <info@michaelhoffer.de>          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as               *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public             *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QWidget>
#include <QLabel>
#include <QPushButton>
#include <QFileDialog>
#include "Canvas.h"
#include "ui_mainwindow.h"
#include <fstream>
#include <string>

#include "helpwindow.h"
#include "ImageArray.h"
#include "ColorTable.h"

/**
 *
 * The mainwindow class.
 *
 * It is derived from the "ui_mainwindow" file created by "uic".
 */
class MainWindow : public QMainWindow, public ValueListener,
			private Ui::MainWindow
{
		Q_OBJECT

	public:
		/**
		 * Constructor
		 */
		MainWindow( QWidget *parent = NULL );

		ImageArray imageData;
		ColorTable colorTable;

		virtual void valueChanged(int value, int x, int y);

		void startPainting();
		void loadFile(std::string fileName);
		void loadCbfFile(std::string fileName);
		std::string fileName;


	protected slots:
		/*
		 * These slots are named by their corresponding
		 * signals. This ensures they are automatically
		 * connected. Se Qt documentation for details.
		 */
	private slots:
		//void on_pbOK_clicked();
		//void on_pbQuit_clicked();
		void on_pbSlope_clicked();
		void on_slopeEdit_returnPressed();

		void on_actionLoadImage_triggered();
		void on_actionUsage_triggered();
		void on_actionAbout_triggered();
		void on_actionQuit_triggered();

		void on_colorSlider_valueChanged();

		//void on_actionQuit_triggered();

	private:
		QString _title;
		HelpWindow *_helpUsageWindow;
		HelpWindow *_helpAboutWindow;
};
#endif /*MAINWINDOW_H*/
